"""ZMC package module."""

# from .connect.connector_context import ConnectorContext as connect

# from .components import *

from .version import __version__

# __all__ = ["__version__", "connect"]
del version
