"""."""

import zmc

print("zmc", dir(zmc))
print()

# print("zmc connect", zmc.connect.__name__)
# print()

print("zmc components", dir(zmc.components))
print()

print("zmc components core", dir(zmc.components.core))
print()

print("zmc version", zmc.version.__version__)
print()

print("zmc utils", dir(zmc.utils))
print()
